@@ -78,13 +78,14 @@ RCvirtualFunProcessing <- setRefClass('RCvirtualFunProcessing',
                                       )
                                       )
 
-RCfunction <- function(f, name = NA, returnCallable = TRUE) {
+RCfunction <- function(f, name = NA, returnCallable = TRUE, check) {
     if(is.na(name)) name <- rcFunLabelMaker()
-    nfm <- nfMethodRC$new(f, name)
+    nfm <- nfMethodRC$new(f, name, check = check)
     if(returnCallable) nfm$generateFunctionObject(keep.nfMethodRC = TRUE) else nfm
 }
 
-is.rcf <- function(x) {
+is.rcf <- function(x, inputIsName = FALSE) {
+    if(inputIsName) x <- get(x)
     if(inherits(x, 'nfMethodRC')) return(TRUE)
     if(is.function(x)) {
         if(is.null(environment(x))) return(FALSE)
