@@ -41,11 +41,20 @@ dimOrLength <- function(obj, scalarize = FALSE) {
 #' y <- matrix(x, nrow = 2)
 #' dim(y)
 #' nimDim(y)
+#'
+#' @export
 nimDim <- function(obj) {
     if(is.null(dim(obj))) return(length(obj))
     return(dim(obj))
 }
 
+#' return the namespace in which a nimbleFunction is being loaded
+#'
+#' \code{nimbleFunction} constructs and evals a reference class definition.  When \code{nimbleFunction} is used in package source code, this can lead to problems finding things due to namespace issues.  Using \code{where = getLoadingNamespace()} in a \code{nimbleFunction} in package source code should solve this problem.
+#'
+#' @details \code{nimbleFunction}s defined in the NIMBLE source code use \code{where = getLoadingNamespace()}.  Please let the NIMBLE developers know if you encounter problems with this.
+#' 
+#' @export
 getLoadingNamespace <- function() {
     if(!is.null(nimbleOptions()$notUsingPackage)) if(nimbleOptions()$notUsingPackage) return(globalenv())
     if(system.file(package = "nimble") == "")
@@ -77,13 +86,15 @@ getNimbleFunctionEnvironment <- function() {
 #' See the User Manual or \code{help(modelValuesBaseClass)} for infomation about modelValues objects
 #'
 #' @examples
-#' mvSpec <- modelValuesSpec(vars = c('a', 'b'),
+#' mvConf <- modelValuesConf(vars = c('a', 'b'),
 #'              types = c('double', 'double'),
 #'              sizes = list(a = 1, b = c(2,2) ) ) 
-#' mv <- modelValues(mvSpec)
+#' mv <- modelValues(mvConf)
 #' as.matrix(mv)
 #' resize(mv, 3)
 #' as.matrix(mv)
+#' 
+#' @export
 resize <- function(container, k) {
     container$resize( as.integer(k) ) 
 }
@@ -99,8 +110,8 @@ resize <- function(container, k) {
 #' See the User Manual or \code{help(modelValuesBaseClass)} for information about modelValues objects 
 #'
 #' @examples
-#'	mvSpec <- modelValuesSpec(vars = 'a', types = 'double', sizes = list(a = 1) )
-#'	mv <- modelValues(mvSpec)
+#'	mvConf <- modelValuesConf(vars = 'a', types = 'double', sizes = list(a = 1) )
+#'	mv <- modelValues(mvConf)
 #'  resize(mv, 10)
 #'	getsize(mv)
 getsize <- function(container) {
@@ -116,3 +127,24 @@ deparse <- function(...) {
       }
 }
 
+
+## creates objects in the parent.frame(), named by names(lst), values are eval(lst[[i]])
+## this is used for creating the conjugate sampler nimble function generators, among other things
+createNamedObjectsFromList <- function(lst, writeToFile = NULL, envir = parent.frame()) {
+    for(i in seq_along(lst)) {
+        objName <- names(lst)[i]
+        obj <- eval(lst[[i]])
+        assign(objName, obj, envir = envir)
+    }
+    if(!is.null(writeToFile)) {
+        write('', file = writeToFile)
+        for(i in seq_along(lst)) {
+            expr <- substitute(VAR <- VALUE, list(VAR = as.name(names(lst)[i]), VALUE = lst[[i]]))
+            deparseExpr <- deparse(expr, control=c())
+            deparseExpr <- gsub('\"', '\'', deparseExpr)
+            write(deparseExpr, file = writeToFile, append = TRUE)
+            write('\n\n\n', file = writeToFile, append = TRUE)
+        }
+    }
+}
+
