@@ -202,7 +202,8 @@ cppProjectClass <- setRefClass('cppProjectClass',
                                            "}")
                                        writeLines(contentLines, con = dynamicRegistrationsCppName)
                                    },
-                                   compileFile = function(names, .useLib = UseLibraryMakevars) {
+                                   compileFile = function(names, showCompilerOutput = nimbleOptions('showCompilerOutput'),
+                                                          .useLib = UseLibraryMakevars) {
                                        cppPermList <- c('RcppUtils.cpp',
                                                         'Utils.cpp',
                                                         'NamedObjects.cpp',
@@ -248,38 +249,35 @@ cppProjectClass <- setRefClass('cppProjectClass',
                                        setwd(dirName)
                                        on.exit(setwd(cur))
 
-                                       showOutput <- getNimbleOption('showCompilerOutput')
-
-
                                        if(is.null(nimbleUserNamespace$sessionSpecificDll)) {
                                            writeDynamicRegistrationsDotCpp(dynamicRegistrationsCppName, dynamicRegistrationsDllName)
                                            ssDllName <- file.path(dirName, paste0(dynamicRegistrationsDllName, .Platform$dynlib.ext))
                                            ssdSHLIBcmd <- paste(file.path(R.home('bin'), 'R'), 'CMD SHLIB', dynamicRegistrationsCppName, '-o', basename(ssDllName))
-                                           if(!showOutput) {
+                                           if(!showCompilerOutput) {
                                                logFile <- paste0("dynamicRegistrations_", format(Sys.time(), "%m_%d_%H_%M_%S"), ".log")
                                                ssdSHLIBcmd <- paste(ssdSHLIBcmd, ">", logFile)
                                                ## Rstudio will fail to run a system() command with show.output.on.console=FALSE if any output is actually directed to the console. Redirecting it to a file seems to cure this.
                                            }
                                            if(isWindows)
-                                               status = system(ssdSHLIBcmd, ignore.stdout = !showOutput, ignore.stderr = !showOutput, show.output.on.console = showOutput)
+                                               status = system(ssdSHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput, show.output.on.console = showCompilerOutput)
                                            else
-                                               status = system(ssdSHLIBcmd, ignore.stdout = !showOutput, ignore.stderr = !showOutput)
+                                               status = system(ssdSHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput)
                                            if(status != 0) 
                                                stop(structure(simpleError("Failed to create the shared library"),
                                                               class = c("SHLIBCreationError", "ShellError", "simpleError", "error", "condition")))
                                            nimbleUserNamespace$sessionSpecificDll <- dyn.load(ssDllName, local = TRUE)
                                        }
 
-                                       if(!showOutput) { 
+                                       if(!showCompilerOutput) { 
                                            logFile <- paste0(names[1], "_", format(Sys.time(), "%m_%d_%H_%M_%S"), ".log")
                                            SHLIBcmd <- paste(SHLIBcmd, ">", logFile)
                                            ## Rstudio will fail to run a system() command with show.output.on.console=FALSE if any output is actually directed to the console. Redirecting it to a file seems to cure this.
                                        }
                                        
                                        if(isWindows)
-                                           status = system(SHLIBcmd, ignore.stdout = !showOutput, ignore.stderr = !showOutput, show.output.on.console = showOutput)
+                                           status = system(SHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput, show.output.on.console = showCompilerOutput)
                                        else
-                                           status = system(SHLIBcmd, ignore.stdout = !showOutput, ignore.stderr = !showOutput)
+                                           status = system(SHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput)
 				       if(status != 0)
                                           stop(structure(simpleError("Failed to create the shared library"),
                                                          class = c("SHLIBCreationError", "ShellError", "simpleError", "error", "condition")))
