@@ -27,6 +27,7 @@
 CmodelValues <- setRefClass(
     Class = 'CmodelValues',
     fields = list(
+        dll = 'ANY',
         extptr = 'ANY',
         extptrCall = 'ANY',
         varNames = 'ANY',
@@ -41,7 +42,7 @@ CmodelValues <- setRefClass(
     			return(NULL)
     		sizeList <- list()
     		for(compName in varNames ){
-    			sizeList[[compName]] <- .Call('getMVsize', componentExtptrs[[compName]])
+    			sizeList[[compName]] <- eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVsize, componentExtptrs[[compName]]))
     		}
     	return(sizeList)
     	}),
@@ -58,30 +59,39 @@ CmodelValues <- setRefClass(
             return(GID_map$expandNodeNames(nodeNames = nodeNames, returnType = returnType, 
                                            flatIndices = flatIndices))
         },
-        initialize = function(buildCall, existingPtr, initialized = FALSE ) {
+        finalizeInternal = function() {
+            finalize()
+            extptr <<- NULL
+        },
+        finalize = function() {
+            nimbleInternalFunctions$nimbleFinalize(extptr)
+        },
+        initialize = function(buildCall, existingPtr, initialized = FALSE, dll) {
             if(missing(existingPtr) ) {
                 if(is.character(buildCall)) {
                     warning("a call to getNativeSymbolInfo with only a name and no DLL")
                 }
-                                        # Are we actually calling this here
-
                 # avoid R CMD check problem with registration
+                ## notice that buildCall is the result of getNativeSymbolInfo using the dll from nimbleProject$instantiateCmodelValues
+                ## only other calling point is from cppInterfaces_models.R, and in that case existingPtr is provided
                 extptr <<- eval(parse(text = ".Call(buildCall)"))
-#                extptr <<- .Call(buildCall) 
+                eval(call('.Call',nimbleUserNamespace$sessionSpecificDll$register_namedObjects_Finalizer, extptr, dll[['handle']], 'modelValues'))
+#                extptr <<- .Call(buildCall)
             }
             else{
                 extptr <<- existingPtr
             }
+            dll <<- dll
             initialized <<- initialized
             if(missing(buildCall) ) 
                 break("Cannot build object without buildCall!")
             extptrCall <<- buildCall
-            varNames <<- .Call(getNativeSymbolInfo('getAvailableNames'), extptr)      
+            varNames <<- eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getAvailableNames, extptr))      
             componentExtptrs <<- vector(mode = 'list', length = length(varNames))
             names(componentExtptrs) <<- varNames
             blankAns <<- componentExtptrs
             for(comp in varNames) 
-                componentExtptrs[[comp]] <<- .Call(getNativeSymbolInfo('getModelObjectPtr'), extptr, comp)
+                componentExtptrs[[comp]] <<- eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getModelObjectPtr, extptr, comp))
                 
             ## .nodePtrs_byGID <<- new('numberedObjects')
             ## GID_map <<- makeMV_GID_Map(.self)
@@ -99,10 +109,10 @@ CmodelValues <- setRefClass(
         },
         resize = function(rows){	
         	for(ptr in componentExtptrs)
-        	jnk <- .Call("setNumListRows", ptr, as.integer(rows), TRUE)
-        	jnk <- .Call('manualSetNRows', extptr, as.integer(rows) )  	
+        	jnk <- eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setNumListRows, ptr, as.integer(rows), TRUE))
+        	jnk <- eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$manualSetNRows, extptr, as.integer(rows)) )  	
         	},
-        getSize = function() {	getCRows(componentExtptrs[[1]])		}
+        getSize = function() {	eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getNRow, componentExtptrs[[1]])) } ## formerly getCRows(componentExtptrs[[1]])		}
         )
     )
 
@@ -118,10 +128,10 @@ setMethod('[', 'CmodelValues',
               	if(is.null(ptr) ) 
               		stop(paste('variable', i, ' not found in modelValues') ) 
               	if(length(j) == 1){
-              		output = .Call('getMVElement', ptr, as.integer(j) )
+              		output = eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVElement, ptr, as.integer(j)) )
               		return(output) 
               		}
-              	output = .Call('getMVElementAsList', ptr, as.integer(j) )
+              	output = eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVElementAsList, ptr, as.integer(j)) )
               	return(output)
               	}
              output <- list() 	
@@ -143,12 +153,12 @@ setMethod('[<-', 'CmodelValues',
 	              		stop(paste('variable', i, ' not found in modelValues') ) 
 					if(length(j) == 1){
 						storage.mode(value) <- 'numeric'
-						.Call('setMVElement', ptr, as.integer(j), value )
+						eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElement, ptr, as.integer(j), value ))
 						return(x)
 					}
 				for(jj in j)
 					storage.mode(value[[jj]]) <- 'numeric'
-				.Call('setMVElementFromList', ptr, value, as.integer(j) )
+				eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElementFromList, ptr, value, as.integer(j)) )
 				return(x)
 				}
 			cmpNames = names(value)
@@ -171,10 +181,10 @@ setMethod('[[', 'CmodelValues',
                   if(is.null(ptr) ) 
                       stop(paste('variable', i, ' not found in modelValues') )
                   if(k == 1){
-                      output = .Call('getMVElement', ptr, as.integer(1) )
+                      output = eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVElement, ptr, as.integer(1)) )
                       return(output) 
                   }
-                  output = .Call('getMVElementAsList', ptr, as.integer(1:k) )
+                  output = eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVElementAsList, ptr, as.integer(1:k)) )
                   return(output)
               }
               output <- list() 	
@@ -193,10 +203,10 @@ setMethod('[[<-', 'CmodelValues',
 	              	if(is.null(ptr) ) 
 	              		stop(paste('variable', i, ' not found in modelValues') ) 
 					if(k == 1){
-						.Call('setMVElement', ptr, as.integer(1), as.numeric(value) )
+						eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElement, ptr, as.integer(1), as.numeric(value) ))
 						return(x)
 					}
-				.Call('setMVElementFromList', ptr, as.numeric(value), as.integer(1:k) )
+				eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElementFromList, ptr, as.numeric(value), as.integer(1:k) ))
 				return(x)
 				}
 			cmpNames = names(value)
