@@ -23,10 +23,10 @@
 ###					Default initialization is assumed that elements are pointing to row 1 of
 ###					the modelValues, but this is not necessary (as long as it is a double pointer to something
 
-getMVptr <- function(rPtr)
-  .Call("getModelValuesPtrFromModel", rPtr)
-getMVName <- function(modelValuePtr)
-  .Call("getMVBuildName", modelValuePtr)
+getMVptr <- function(rPtr, dll)
+  eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getModelValuesPtrFromModel, rPtr))
+getMVName <- function(modelValuePtr, dll)
+  eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVBuildName, modelValuePtr))
 
 # for now export this as R<3.1.2 give warnings if don't
 
@@ -38,6 +38,8 @@ getMVName <- function(modelValuePtr)
 CmodelBaseClass <- setRefClass('CmodelBaseClass',
                                contains = 'modelBaseClass',
                                fields = list(
+                                   .basePtr = 'ANY',
+                                   dll = 'ANY',
                                    Rmodel = 'ANY',
                                    cppNames = 'ANY',
                                    cppCopyTypes = 'ANY', ## At the given moment these will all be 'numeric', but the system allows more flexibility
@@ -53,6 +55,30 @@ CmodelBaseClass <- setRefClass('CmodelBaseClass',
                                    show = function() {
                                        cat('CmodelBaseClass object\n')
                                    },
+                                   finalizeInternal = function() {
+                                       for(vn in cppNames) {
+                                           vPtrName <- paste(".", vn, "_Ptr", sep = "")
+                                           assign(vPtrName, NULL, inherits = TRUE)
+                                       }
+                                       finalize()
+                                       .basePtr <<- NULL
+                                       .nodeFxnPointers_byDeclID <<- NULL
+                                       nimbleProject <<- NULL
+                                   },
+                                   finalize = function() {
+                                       for(i in ls(Rmodel$nodes)) {
+                                           if(is.null(nodes[[i]])) next
+                                           if(is.list(nodes[[i]]))
+                                               nodes[[i]][[1]]$finalizeInstance(nodes[[i]][[2]])
+                                           else
+                                               nodes[[i]]$finalize()
+                                           nodes[[i]] <<- NULL
+                                       }
+                                       if(!is.null(.nodeFxnPointers_byDeclID))
+                                           .nodeFxnPointers_byDeclID$finalize()
+                                       if(!is.null(.basePtr))
+                                           nimbleInternalFunctions$nimbleFinalize(.basePtr)
+                                   },
                                    setModel = function(model) {
                                        ## This is creating a circular reference, so be careful with show(), and with Rstudio
                                        Rmodel <<- model 
@@ -85,7 +111,7 @@ CmodelBaseClass <- setRefClass('CmodelBaseClass',
                                        names(nodeFunctions) <<- names(Rmodel$nodeFunctions)
                                        
                                        ##.nodeFxnPointers_byGID <<- new('numberedObjects')
-                                       .nodeFxnPointers_byDeclID <<- new('numberedObjects') 
+                                       .nodeFxnPointers_byDeclID <<- new('numberedObjects', dll = dll) 
                                        ##maxID = length(modelDef$maps$graphIDs)
                                        maxID = length(modelDef$declInfo)
                                        ##.nodeFxnPointers_byGID$resize(maxID)
@@ -145,16 +171,16 @@ makeModelCppCopyTypes <- function(symTab) {
 
 
 makeModelBindingFields <- function(symTab) {
-  fieldList = list(.basePtr = "ANY", .modelValues_Ptr = "ANY", .DUMMY = "ANY")  
+  fieldList = list(.modelValues_Ptr = "ANY", .DUMMY = "ANY")  
   vNames = names(symTab$symbols)
   for(vn in vNames){
     ptrName = paste(".", vn, "_Ptr", sep = "")
     fieldList[[ptrName]] <- "ANY"
     eval(substitute( fieldList$VARNAME <- function(x){
       if(missing(x) ) 
-        nimbleInternalFunctions$getNimValues(VPTR, 2)
-      else
-        nimbleInternalFunctions$setNimValues(VPTR, x, 2, allowResize = FALSE)
+        nimbleInternalFunctions$getNimValues(VPTR, 2, dll = dll)
+      else 
+        nimbleInternalFunctions$setNimValues(VPTR, x, 2, allowResize = FALSE, dll = dll)
     }, list(VPTR = as.name(ptrName), VARNAME = vn) ) )
   }
   return(fieldList)
@@ -188,13 +214,15 @@ buildModelInterface <- function(refName, compiledModel, basePtrCall, project = N
                                                 isDataEnv <<- new.env()
                                                 classEnvironment <<- new.env()
                                                 
-                                                callSuper()
+                                                callSuper(dll = dll, ...)
 
-                                                # avoid R CMD check problem with registration
+                                        # avoid R CMD check problem with registration
+                                                ## notice that the following line appears a few lines up:basePtrCall = getNativeSymbolInfo(basePtrCall, dll)
                                                 .basePtr <<- eval(parse(text = ".Call(basePtrCall)"))
+                                                eval(call('.Call',nimbleUserNamespace$sessionSpecificDll$register_namedObjects_Finalizer, .basePtr, dll[['handle']], model$name))
                                                 # .basePtr <<- .Call(BPTRCALL)
-                                                .modelValues_Ptr <<- nimbleInternalFunctions$getMVptr(.basePtr)
-                                                defaultModelValues <<- nimbleInternalFunctions$CmodelValues$new(existingPtr = .modelValues_Ptr, buildCall = nimbleInternalFunctions$getMVName(.modelValues_Ptr), initialized = TRUE )
+                                                .modelValues_Ptr <<- nimbleInternalFunctions$getMVptr(.basePtr, dll = dll)
+                                                defaultModelValues <<- nimbleInternalFunctions$CmodelValues$new(existingPtr = .modelValues_Ptr, buildCall = nimbleInternalFunctions$getMVName(.modelValues_Ptr, dll), initialized = TRUE, dll = dll )
                                                 modelDef <<- model$modelDef
                                                 graph <<- model$graph
                                                 vars <<- model$vars
@@ -202,13 +230,13 @@ buildModelInterface <- function(refName, compiledModel, basePtrCall, project = N
                                                 nimbleProject <<- defaults$project
                                                 for(v in ls(model$isDataEnv)) isDataEnv[[v]] <<- model$isDataEnv[[v]]
                                                 setData(modelDef$constantsList, warnAboutMissingNames = FALSE)
-                                                cppNames <<- .Call("getAvailableNames", .basePtr) ## or could get this from R objects
+                                                cppNames <<- eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getAvailableNames, .basePtr)) ## or could get this from R objects
                                                 cppCopyTypes <<- defaults$cppCT
                                                 compiledModel <<- defaults$cm
                                                 for(vn in cppNames)
                                                     {
                                                         vPtrName <- paste(".", vn, "_Ptr", sep = "")
-                                                     	.self[[vPtrName]] <<- nimbleInternalFunctions$newObjElementPtr(.basePtr, vn)
+                                                     	.self[[vPtrName]] <<- nimbleInternalFunctions$newObjElementPtr(.basePtr, vn, dll = dll)
                                                     }      
                                                 if(!missing(model)) {
                                                     setModel(model)
